import { dRequest, TokenList } from './DataTypes'
import   hljs        from 'highlight.js/lib/core'
import   typescript  from 'highlight.js/lib/languages/typescript'
import   Lexer       from './Lexer'
import   Parser      from './Parser'

/**
 *  **Code Generator**
 * 
 *  Receives the codes and languages types to return formatted React standard 
 * contents using JSX(TSX). 
 * 
 */

export function
CodeGen(props: dRequest) {
    hljs.registerLanguage('typescript', typescript );

    // Default Highlight.JS lib request.
    const hljsBuffer  = hljs.highlight( props.code, { language: props.lang }).value

    // Create a Lexer Class to tokenize HLJS return
    const SpanLexer = new Lexer({ txt: hljsBuffer })

    // Create a Parser to process all Span token generated from HLJS lib
    const SpanParser  = new Parser({ Lexer: SpanLexer })

    // Initialize the Parser Engine
    SpanParser.exec()

    // 
    const response : JSX.Element[] = []
    let   fragment     : JSX.Element = <></>
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    let buffer: any
    let count = 0
 
    /**
     * Process only Span with class text contents or any content inside the tag.  
     */
    while(SpanParser.SpanASQ.remove()) {
        buffer = SpanParser.SpanASQ.getLastRemoved()
        // Only Span
        if( buffer.token === TokenList._span ) {
            fragment = (
                <span 
                    key={ window.crypto.randomUUID() }
                    className={buffer.txtContent}>
                    {buffer.tagContent}
                </span>
            )
        }
        // Any other text contents
        if( buffer.token === TokenList._content)
            fragment = <span key={count++}>{buffer.tagContent}</span>

        response.push(fragment)
    }
    return (
        <code className={'hljs ' + props.lang + ' ' + props.className} >
            {response}
        </code>)
}