/**
 * Data for request the React component from Span Parser
 * ```Typescript
 * TYPE
 *   code `string` //Array containing your code for the conversion
 *   lang `string` //Computer Language compatible with Highligh.JS
 * ```
 * ---
 * **Warning**
 * 
 * The application does not use the auto detect function provided by native lib. 
 */
export
type dRequest = {
    code      : string,
    lang      : string
    className?: string
}

/**
 * A simple token list of the most important named token, processed inside the 
 * lexical analyzer ( HljsToReact, Lex, Lexer )
 */
export 
enum TokenList {
    _undef       =  0,
    _eof         = -1,
    _span        = -2,
    _class       = -3,
    _tag_content = -4,
    _content     = -5
}


/**
 * Only use three flags to map the module. 
 */
export
enum LogFlagList {
    _ok            =  0,
    _Lexer_error = -1,
    _parser_error  = -2
}

/**
 * Node data struct using class to queue the span data or contents wrapping by 
 * <span> CONTENT </span> 
 * ---
 * ```typescript
 * CLASS
 *   token     : number // TokenList or ASCII number code with charCodeAt
 *   tagContent: string // Span tag content ...> string </... | <...
 *   txtContent: string // Class text content class=" string "
 *   child     : class  // SpanParseNode as a child 
 * ```
 */
export class
SpanParseNode {
    token       : number         
    tagContent ?: string 
    txtContent ?: string
    child       : SpanParseNode | null = null

    constructor (token: number, tagContent ?: string, txtContent ?: string) {
      this.token      = token
      this.tagContent = tagContent
      this.txtContent = txtContent
    }
}