/* eslint-disable @typescript-eslint/no-inferrable-types */
import { InsideConsoleLog } from './Log'
import { TokenList }        from './DataTypes'

/**
 *  **Span Lexer Class**
 * 
 * For  using to request and response token.  The simple lexical analyzer class. 
 * Return  a few  token types,  but necessary to  implement the  abstract parser 
 * queue, in Span Parser.
 * ---
 * ```typescript
 * CLASS
 *   Log              : InsideConsoleLog // Only verbose Log in terminal
 *   next             : number           // Next char request by parser
 *   identifier       : string           // Span or class identifier
 *   txt              : string           // Buffer text content 
 *   size             : string           // Size of code string, as a EOF.
 *   private last_char: string           // last_char before next
 * ```
 */

export default class
Lexer {
    Log              : InsideConsoleLog
    next             : number = 0
    identifier       : string = ''
    txt              : string
    size             : number
    private last_char: string  = ' '

    /**
     * 
     * ```typescript
     * constructor 
     *   >> txt: string
     * ```
     */
    constructor({ txt }: { txt: string }) {
        this.txt = txt
        this.size= txt.length
        this.Log = new InsideConsoleLog()
    }

    /**
     * 
     * ```typescript
     * next_char : getter 
     *   << next_char
     * ```
     */   
    get 
    next_char(): string {
        return this.txt[this.next++]
    }

    /**
     * 
     * ```typescript
     * lastChar 
     *   << last_char
     * ```
     */
    lastChar() : string {
        return this.last_char
    }

    /**
     * 
     * ```typescript
     * content: getter 
     *   << buffer: string
     * ```
     */
    get
    content() {
        let buffer  = ''

        this.last_char = this.next_char
        while(this.last_char !== '<' &&
              this.last_char !== '"' &&
              this.next <= this.size) {
            buffer += this.last_char
            this.last_char = this.next_char
        }

        console.log('BUUFER: '+ this.lastChar())
        this.next--
        return buffer
    }

    /**
     * Return if the code text size ended. As a EOF 
     * ```typescript
     * checkEof 
     *   >> txt: string
     * ```
     */
    checkEof(): boolean {
        if( this.next >= this.size ) {
            return false
        }
        return true
    }

    /**
     * The monst important method inside Lexer Class. It is responsible for get
     * the specific token for using in Span Parser.
     * ```typescript
     * token 
     *   << TokenList: number
     * ```
     */
    token(): number {
        this.Log.initLexer()

        this.last_char = this.txt[this.next++]

        // Whitespace scape
        while( this.space(this.last_char) )
            this.last_char = this.next_char

        // [a-zA-Z] only
        if( this.alpha(this.last_char) ) {
            this.identifier = ''
            do {
                this.identifier += this.last_char
                this.last_char  = this.next_char
            }while( this.alpha(this.last_char))
            this.next--
            console.log('Identifier return: ' + this.identifier )

            if( this.identifier === 'span' )
                return TokenList._span

            if( this.identifier === 'class' )
                return TokenList._class
            return TokenList._undef
        }

        // Return itself ASCII code as a token
        if( this.openTagSimbol(this.last_char) ||
            this.closeTagSimbol(this.last_char)||
            this.checkLetter(this.last_char, '"') ||
            this.checkLetter(this.last_char, '=' ) ||
            this.checkLetter(this.last_char, '/' ) )
            return this.last_char.charCodeAt(0)
        return TokenList._undef
    }

    /**
     * Check all whitespace code
     * ```typescript
     * space
     *   >> letter: string 
     *   << boolean
     * ```
     */
    space( letter: string ): boolean {
        const space = 
        letter === ' '
        || letter === '\u00a0'
        || letter === '\u1680'
        || letter === '\u200a'
        || letter === '\u2028'
        || letter === '\u2029'
        || letter === '\ufeff'
        
        letter = space  === true ? 'whitespace' : 
                 letter === ''   ? 'empty'      : letter

        console.log('Letter( ' + letter + ' ).test( whitespace.All )')
        return space ? true : false
    }

    /**
     * Check [a-zA-Z]
     * ```typescript
     * alpha
     *   >> letter: string 
     *   << buffer: boolean
     * ```
     */
    alpha( letter: string ): boolean {
        const buffer = /^[a-zA-Z]$/.test( letter )
        console.log( 'Alpha letter: ' + ( buffer ? 'letter' : 'not found' ) )
        return buffer
    }
    
    /**
     * Compares the inputs
     * ```typescript
     * checkLetter
     *   >> letter    : string
     *   >> check_this: string 
     *   << boolean
     * ```
     */
    checkLetter( letter: string, check_this: string ) {
        console.log( 'Letter( ' + letter + ' ).test( ' + check_this + ' )' )
        return letter === check_this ? true : false
    }

    /**
     * Compares the inputs
     * ```typescript
     * openTagSimbol
     *   >> letter    : string
     *   >> check_this: string 
     *   << boolean
     * ```
     */    
    openTagSimbol( letter: string ) {
        return this.checkLetter( letter, '<' )
    }

    /**
     * Compares the inputs
     * ```typescript
     * closeTagSimbol
     *   >> letter    : string
     *   >> check_this: string 
     *   << boolean
     * ```
     */
    closeTagSimbol( letter: string ) {
        return this.checkLetter( letter, '>' )
    }
}