import { TokenList } from "../DataTypes"
import Lexer from "../Lexer"

describe( 'Simples Span Tag Lexer', () => {
    const code = 'text init extern  '
                 '<span class="text inside class">'
                 ' text inside tag  </span>'
                 ' text close extern'
    const lexer = new Lexer({txt: code})
    
    const whitespace = ' '
    const ucletter   = 'M'
    const lcletter   = 'w'
    const number     = '1'
    const empty      = ''
    const name       = 'MW'
    const simbol     = '>'

    it( 'token() : return correct token', () => {
        lexer.txt = '< span class = " / >'
        expect( lexer.token()).toBe('<'.charCodeAt(0) )
        expect( lexer.token()).toBe(TokenList._span   )
        expect( lexer.token()).toBe(TokenList._class  )
        expect( lexer.token()).toBe('='.charCodeAt(0) )
        expect( lexer.token()).toBe('"'.charCodeAt(0) )
        expect( lexer.token()).toBe('/'.charCodeAt(0) )
        expect( lexer.token()).toBe('>'.charCodeAt(0) )
    })

    it( 'lastChar() ', () => {
        const txt = '="'
        const tmp_lexer = new Lexer({txt: txt})
        tmp_lexer.token()
        expect( tmp_lexer.lastChar()).toBe('=')
        expect( tmp_lexer.lastChar()).not.toBe('"')
    })

    it( 'content : return all content', () => {
        let txt = 'content text<'
        let tmp_lexer = new Lexer({txt: txt})
        expect( tmp_lexer.content ).toBe('content text')

        txt = 'content text"'
        tmp_lexer = new Lexer({txt: txt})
        expect( tmp_lexer.content ).toBe('content text')
    })

    it( 'next_char : return only the next found char', () => {
        const txt = 'MW'
        const tmp_lexer = new Lexer({txt: txt})
        expect( tmp_lexer.next_char ).toBe('M')
        expect( tmp_lexer.next_char ).not.toBe('M')
    })

    it( 'space() : detect only whitespace', () => {
        expect( lexer.space( whitespace)).toBe(true)
        expect( lexer.space( ucletter  )).toBe(false)
        expect( lexer.space( empty     )).toBe(false)
    })

    it( 'alpha() : detect only alpha',  () => {
        expect( lexer.alpha( ucletter  )).toBe(true)
        expect( lexer.alpha( lcletter  )).toBe(true)
        expect( lexer.alpha( name      )).toBe(false)
        expect( lexer.alpha( empty     )).toBe(false)
        expect( lexer.alpha( simbol    )).toBe(false)
        expect( lexer.alpha( number    )).toBe(false)
        expect( lexer.alpha( whitespace)).toBe(false)
    })

    it( 'checkLetter() : compare all character type', () => {
        expect( lexer.checkLetter( name      , name         )).toBe(true)
        expect( lexer.checkLetter( empty     , empty        )).toBe(true)
        expect( lexer.checkLetter( whitespace, whitespace   )).toBe(true)
        expect( lexer.checkLetter( whitespace, empty        )).toBe(false)
    })

    it( 'openTagSimbol() : detect only \'<\'', () => {
        expect( lexer.openTagSimbol('<')).toBe(true)
        expect( lexer.openTagSimbol('>')).toBe(false)
    })

    it( 'closeTagSimbol() : detect only \'>\'', () => {
        expect( lexer.closeTagSimbol('>')).toBe(true)
        expect( lexer.closeTagSimbol('<')).toBe(false)
    })

    it( 'checkEOF : If the code text return last position from array', () => {
        const txt = 'Makoto World'
        const tmp_lexer = new Lexer({txt: txt})
        const size = txt.length
        tmp_lexer.next = size
        expect(tmp_lexer.checkEof()).toBe(false)

        tmp_lexer.next++
        expect(tmp_lexer.checkEof()).toBe(false)

        tmp_lexer.next = size - 1
        expect(tmp_lexer.checkEof()).toBe(true)
    })
})